function M = hansenm(ell,m,r,theta,phi,k,order)
%HANSENM Hansen solution M of the spherical vectorial wave equation 
%   M = HANSENM(ELL,M,R,THETA,PHI,K,ORDER) computes Hansen solution M
%       of the vectorial wave equation for degree ell and order m 
%       (-ell <= m <= ell) at point (r,theta,phi) in spherical coordinates
%       k is the wave number involved in the vectorial wave equation 
%       order (= 1 or 2) indicates the behavior of the solution in the
%       radial direction and correspond either to a spherical Bessel
%       function of the first kind (order=1) or to a spherical Bessel
%       function of the second kind (order=2)
%
% Author: Stephane BALAC, IRMAR, CNRS, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 10 september 2017
% Program distributed under the GNU Lesser General Public License (LGPL)

I=sqrt(-1);
x=k*r;

if m~=0
    C=(-1)^m*sqrt((2*ell+1)/(8*pi));
else
    C=sqrt((2*ell+1)/(4*pi));
end
% Radial dependency
if order==1
    z=sbesselj(ell,x);
    dz=dsbesselj(ell,x);
elseif order==2
    z=sbessely(ell,x);
    dz=dsbessely(ell,x);
else
    error('HANSENM : undefined order for spherical Bessel function');
end
% angular dependency
if theta~=0
    M(1,:) = 0;
    M(2,:) = (I*m*C./sin(theta)).*z.*slegend(ell,m,cos(theta)).*exp(I*m*phi);
    M(3,:) = C*sin(theta).*z.*dslegend(ell,m,cos(theta)).*exp(I*m*phi);
else
    M(1,:) = 0;
    M(2,:) = 0;
    M(3,:) = C*sin(theta).*z.*dslegend(ell,m,cos(theta)).*exp(I*m*phi);
end
end