function N = hansenn(ell,m,r,theta,phi,k,order)
%HANSENN Hansen solution N of the spherical vectorial wave equation 
%   N = HANSENN(ELL,M,R,THETA,PHI,K,ORDER) computes Hansen solution N
%       of the vectorial wave equation for degree ELL and order M 
%       (-ELL <= M <= ELL) at point (r,theta,phi) in spherical coordinates
%       k is the wave number involved in the vectorial wave equation 
%       order (= 1 or 2) indicates the behavior of the solution in the
%       radial direction and correspond either to a spherical Bessel
%       function of the first kind (order=1) or  to a spherical Bessel
%       function of the second kind (order=2)
%
% Author: Stephane BALAC, IRMAR, CNRS, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 10 september 2017
% Program distributed under the GNU Lesser General Public License (LGPL)

I=sqrt(-1);
x=k*r;

if m~=0
    C=(-1)^m*sqrt((2*ell+1)/(8*pi));
else
    C=sqrt((2*ell+1)/(4*pi));
end
% radial dependency
if order==1
    z=sbesselj(ell,x);
    dz=dsbesselj(ell,x);
elseif order==2
    z=sbessely(ell,x);
    dz=dsbessely(ell,x);
else
    error('HANSENN : undefined order for spherical Bessel function');
end
% angular dependency
if theta~=0 && r~=0
    N(1,:) = (C*ell*(ell+1)./x).*z.*slegend(ell,m,cos(theta)).*exp(I*m*phi);
    N(2,:) = -(C./x).*(z+x*dz).*sin(theta).*dslegend(ell,m,cos(theta)).*exp(I*m*phi);
    N(3,:) = (I*m*C./(x*sin(theta))).*(z+x*dz)*slegend(ell,m,cos(theta)).*exp(I*m*phi);
elseif theta==0 && r~=0
    N(1,:) = (C*ell*(ell+1)./x).*z.*slegend(ell,m,cos(theta)).*exp(I*m*phi);
    N(2,:) = 0;
    N(3,:) = 0;
else
    N(1,:) = 0;
    N(2,:) = 0;
    N(3,:) = 0;
end

end

