function [E,B]=modefield(ell,m,r,theta,phi,tetm,lambda,R,N)
% MODEFIELD : computes the electromagnetic field (electric field and
% magnetic induction) of a given mode defined by its mode index (ell,m) and
% resonance wavelength lambda

% VARIABLES IN :
% ell : first resonance index
% m : second resonance index
% r : radial coordinate of the points where the EM field is to be computed
% theta : polar coordinate of the points where the EM field is to be computed
% phi : azimuthal coordinate of the points where the EM field is to be computed
% tetm : selector for TE or TM modes
% lambda : wavelength in mu-m
% R : micro-sphere radius
% N : optical index of the cavity
%
% VARIABLES OUT:
% E : components of the electric field at (r,theta,phi)
% B : components of the magnetic induction at (r,theta,phi)
%
% Author: Stephane BALAC, IRMAR, CNRS, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 10 september 2017
% Program distributed under the GNU Lesser General Public License (LGPL)


cc=299792458E+6;  % Speed of light in micro-m/s

k0=2*pi/lambda;
k=k0*N;
omega=2*pi*cc/lambda;

if strcmp(tetm,'TE') % TE mode
    cste=-besselj(ell+1/2,k*R)/(sqrt(N)*bessely(ell+1/2,k0*R));
    for i=1:length(r)
        ri=r(i);
        if ri<=R
            Mhans=hansenm(ell,m,r(i),theta,phi,k,1);
            E(i)=norm(Mhans);
            Nhans=(1i*k/omega)*hansenn(ell,m,r(i),theta,phi,k,1);
            B(i)=norm(Nhans);
        else
            Mhans=hansenm(ell,m,r(i),theta,phi,k0,2);
            E(i)=norm(cste*Mhans);
            Nhans=(1i*k0/omega)*hansenn(ell,m,r(i),theta,phi,k0,2);
            B(i)=norm(cste*Nhans);
        end
    end
else%TM mode
    for i=1:length(r)
        cste=-sqrt(N)*besselj(ell+1/2,k*R)/bessely(ell+1/2,k0*R);
        ri=r(i);
        if ri<=R
            Mhans=(1i*k/omega)*hansenm(ell,m,r(i),theta,phi,k,1);
            B(i)=norm(Mhans);
            Nhans=hansenn(ell,m,r(i),theta,phi,k,1);
            E(i)=norm(Nhans);
        else
            Mhans=(1i*k0/omega)*hansenm(ell,m,r(i),theta,phi,k0,2);
            B(i)=norm(cste*Mhans);
            Nhans=hansenn(ell,m,r(i),theta,phi,k0,2);
            E(i)=norm(cste*Nhans);
        end
    end
end
