%QFACTN COMPUTES THE QUALITY FACTOR
%  QFACTN computes, for a given mode index and a given approximate value
%   of the resonance wavelength the quality factor of the mode.
%  This requires the computation of the exact complex value of the resonance
%   wavelength by solving the exact modal equation
%  The modal equation is solved by Delves-Lyness Method

% Author: Stephane BALAC, IRMAR, CNRS, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 10 september 2017
% Program distributed under the GNU Lesser General Public License (LGPL)

clear 
close all
expert_mode=0; % set to 1 for using the program in expert mode
%
% Data reading 
%
R=input('Micro-sphere radius [micro-m] = ');
N=input('Optical index of the micro-sphere = ');
TEM=input('TE or TM mode ? (TE/TM) : ','s');
if strcmp(TEM,'te')
    TEM='TE';
end
if strcmp(TEM,'TE')
    P=N;
else
    P=1/N;
end
ell=input('Mode index ell = ');
lambda0=input('Approximated real value of the resonance wavelength [nm] = ');
lambda0=1E-3*lambda0;  % units conversion in micron-m 
k0=2*pi./lambda0; % wave number in vacuum
z0=k0*R;
if expert_mode==0
    rlamb=1E-3; % initial radius in micro-meter
else
    rlamb=input('Radius of the disk where the root is located in nm = ');
    rlamb=rlamb*1E-3; % conversion in micro-meter
end

% We check the number of zeros in the disk D(z0,r)
nbofz=0; % initialization of the number of zeros
ntry=1;
r=abs(2*pi*R*rlamb/(lambda0^2)); % adimensionalized radius
while nbofz~=1
    mquad=1000; % number of nodes of the quadrature formula
    tj=[1:mquad]./mquad;
    z=z0+r*exp(2*pi*1i*tj);
    fz=r*intgdl(ell,z,P,N);
    s0=(1/mquad)*sum(fz.*exp(2*pi*1i*tj));
    nbofz=round(abs(s0));
    if nbofz==1
        break;
    elseif nbofz>1
        r=0.5*r; % we divide by 2 the radius
        disp(['We found ',num2str(nbofz),' roots inside the disk']);
        disp(['We divide by 2 the radius and proceed ...']);
    elseif nbofz==0
        r=2*r; % we increase by 2 the radius
        disp(['We found no roots inside the disk']);
        disp(['We multiply by 2 the radius and proceed ...']);
    else 
        error([' This case should not happen !!!' ...
               'It may have too many roots in the disk and the problem '...
               'becomes ill conditioned']);
    end
    ntry=ntry+1;
    if ntry>20
        disp('Too many attempts. Reconsider the value provided for the radius');
        return;
    end
end
disp(' ')
rlamb=abs(2*pi*r/(k0^2*R)); % conversion of the radius in wavelength
disp(['One root (wavelength resonance) of the modal equation located in '...
    'the disk centered at ',...
    num2str(lambda0*1E3),' nm with radius ',num2str(rlamb*1E3),' nm']);

mquad=1000;
ntry=1;
if expert_mode==0
    tol=1E-9; % tolerance for the quadrature
else
    tol=input(' Tolerance for the quadrature formula = ');
end

s1ho=(1/mquad)*sum(fz.*exp(4*pi*1i*tj));
err=1;
disp(' ')
while (err>tol && ntry < 10)
    mquad=2*mquad; % number of nodes of the quadrature formula
    ntry=ntry+1;
    tj=[1:mquad]./mquad;
    z=z0+r*exp(2*pi*1i*tj);
    fz=r*intgdl(ell,z,P,N);
    s1h=(1/mquad)*sum(fz.*exp(4*pi*1i*tj));
    err=abs((s1h-s1ho)/s1h);
end
s1=z0+r*s1h;
lambda= 2*pi*R./s1;

Q=abs(real(lambda)/imag(lambda));
disp(['Complex resonance wavelength for the mode in [nm] = ',...
      num2str(lambda*1E3,16)]);
formatSpec = '%s  %6.3e';
tmp=sprintf(formatSpec,'Quality factor for the mode = ',Q);
disp(tmp)


