%VOLMOD WHISPERING GALLERY MODES VOLUME COMPUTATION
%   VOLMOD computes the volume of a whispering gallery TE or TM modes
%          in a micro-sphere
%
% Author: Stephane BALAC, IRMAR, CNRS, University of Rennes, France
% contact: stephane.balac@univ-rennes1.fr
% Last modification: 10 september 2017
% Program distributed under the GNU Lesser General Public License (LGPL)


clear
close all
expert_mode=0; % set to 1 for using the programm in expert mode
I=sqrt(-1);
mu0=4*pi*1E-1; % magnetic permeability of vacuum in H/micro-m
%
% Data reading
%
R=input('Micro-sphere radius [micro-m] = ');
N=input('Optical index of the micro-sphere = ');
TEM=input('TE or TM mode ? (TE/TM) : ','s');
if strcmp(TEM,'te')
    TEM='TE';
end
lambda=input('Wavelength [nm] = ');
lambda=1E-3*lambda;  % units conversion in micro-m
k0=2*pi/lambda;
k=N*k0;
ell=input('Mode number ell = ');
m=input('Mode number m = ');
if expert_mode
    L=input('Outside integration limit L = ');
else
    L=1.2*R;
end
t=cputime;
%
% I - Computation of the un-normalized mode volume
%
if strcmp(TEM,'TE')
    Cste=ell*(ell+1)*pi/(4*k);
else
    Cste=ell*(ell+1)*pi/(4*k);
end
% Intermediate computation of Bessel functions
YellR=bessely(ell+1/2,k0*R);
YellpR=bessely(ell+3/2,k0*R);
YellmR=bessely(ell-1/2,k0*R);
JellR=besselj(ell+1/2,k*R);
JellpR=besselj(ell+3/2,k*R);
JellmR=besselj(ell-1/2,k*R);
YellL=bessely(ell+1/2,k0*L);
YellpL=bessely(ell+3/2,k0*L);
YellmL=bessely(ell-1/2,k0*L);
voli=Cste*R^2*(JellR^2-JellmR*JellpR);
volo=JellR^2*(L^2*(YellL/YellR)^2-L^2*(YellmL/YellR)*(YellpL/YellR)-...
    R^2+R^2*(YellmR/YellR)*(YellpR/YellR));
if  strcmp(TEM,'TE')
    volo=(Cste/N^2)*volo;
else
    volo=Cste*volo;
end
vol=voli+volo;
%
% II - Computation of the normalization constant
%
% II - A] - Case of TE modes
%

if strcmp(TEM,'TE')
    
    % II-A-1) Contribution of the radial dependency to the normalization constant
    %
    % initial guess fo the first zero of j_ell'
    a1=(ell+1/2)+0.8086165*(ell+1/2)^(1/3)-0.236680*(ell+1/2)^(-1/3)...
        -0.20736*(ell+1/2)^(-1)+0.0233*(ell+1/2)^(-5/3);
    % computes the first zero of j_ell' with fsolve
    options = optimset('Display','off','TolFun',1e-10,'TolX',1e-10);
    f = @(x)vshnor(x,ell);
    a2=fsolve(f,a1,options);
    maxr=(pi/(2*a2))*besselj(ell+1/2,a2)^2;
    %
    % II-A-1) computes the maximal value of the vector spherical harmonic X_ell^m
    %
    fobjte=@(theta) - ((m^2+1)*slegend(ell,m,cos(theta)).^2+...
        (ell-m+1)*(ell+m+1)*slegend(ell+1,m,cos(theta)).^2-...
        2*sqrt((ell-m+1)*(ell+m+1))*slegend(ell,m,cos(theta)).*...
        slegend(ell+1,m,cos(theta)) ) ./ sin(theta).^2;
    foptte=@(x) fopte(cos(x),ell,m);
    h=pi/500;
    theta=h:h:pi-h;
    y=fobjte(theta);
    [vmin,j0] = min(y);
    vmax=-vmin;
    % When Optimization toolbox is available, accuracy of the minimum of fobjte is
    % improve by using the Nonlinear programming solver fminunc
    fval=vmin;
    opt_tlbx=0;
    if infolog('MATLAB') &&  infolog('Optimization Toolbox')
        disp('----------------------------------------------------------------------')
        opt_tlbx=1;
        options = optimoptions('fminunc','Algorithm','trust-region','GradObj','on',...
        'TolX',1E-10,'Tolfun',1E-10);
        [x0,fval]=fminunc(foptte,theta(j0),options);
        vmax=-fval;
        disp('----------------------------------------------------------------------')
    elseif   infolog('Octave') && infolog('optim')
        disp('----------------------------------------------------------------------')
        opt_tlbx=1;
        [x0,fval]=fminunc(foptte,theta(j0));
        vmax=-fval;
        disp('----------------------------------------------------------------------')
    end
    if abs(vmin-fval)/abs(vmin) > 5E-2 ||  opt_tlbx==0
        if  opt_tlbx==1
            disp('fminunc probably reach a local minimum')
        else
            disp(' ');
            disp('No optimization toolbox found');
        end
        disp('Maximization by values comparison launched')
        h=pi/10000;
        theta=h:h:pi-h;
        y=fobjte(theta);
        [vmin,j0] = min(y);
        vmax=-vmin;
    end
        
    if m~=0
        vmax=vmax.*(2*ell+1)./(8*pi);
    else
        vmax=vmax.*(2*ell+1)./(4*pi);
    end
    % Square of the maximum valued of the electric field inside the sphere
    EMmax2=maxr*vmax;
    % Volume of the mode
    vol=vol/EMmax2;
    disp([' Mode volume (in micron.m^3)= ',num2str(vol)]);
    disp(['CPU time for mode volume computations = ', num2str(cputime-t,4),' sec.'])
    
else
    %
    % II - B] - Case of TM modes
    %
    fobjtm=@(r,theta)- ( (0.5*pi*(ell*(ell+1))^2./(k*r)).*(besselj(ell+0.5,k*r).^2) ...
        .*(slegend(ell,m,cos(theta)).^2) ...
        + ( -ell*sqrt(0.5*pi./(k*r)).*besselj(ell+0.5,k*r) ...
        +sqrt(0.5*pi*k*r).*besselj(ell-0.5,k*r) ).^2 .* ( ...
        (m^2+1)*slegend(ell,m,cos(theta)).^2+(ell-m+1)*(ell+m+1)*slegend(ell+1,m,cos(theta)).^2 ...
        -2*sqrt((ell-m+1)*(ell+m+1))*slegend(ell,m,cos(theta)).*slegend(ell+1,m,cos(theta)) ...
        )./sin(theta).^2 )./(k^2*r.^2);
    
    ht=pi/200;
    theta=ht:ht:pi-ht;
    hr=R/200;
    r=hr:hr:R-hr;
    for i=1:length(r)
        y(i,:)=fobjtm(r(i),theta);
    end
    [vmin,jmin] = min(y(:));
    [j_row, j_col] = ind2sub(size(y),jmin);
    
    fopttm=@(x) foptm([x(1),cos(x(2))],k,ell,m);
    % When Optimization toolbox is available, accuracy of the minimum of fobjtm is
    % improve by using the Nonlinear programming solver fminunc
    fval=vmin;
    opt_tlbx=0;
    if infolog('MATLAB') &&  infolog('Optimization Toolbox')
        disp('----------------------------------------------------------------------')
        opt_tlbx=1;
        options = optimoptions('fminunc','Algorithm','trust-region','GradObj','on',...
        'TolX',1E-10,'Tolfun',1E-10);
        [x0,fval]=fminunc(fopttm,[r(j_row),theta(j_col)],options);
        vmax=-fval;
        disp('----------------------------------------------------------------------')
    elseif   infolog('Octave') && infolog('optim')
        disp('----------------------------------------------------------------------')
        %       [x0,fval]=fminunc(foptte,theta(j0));
        opt_tlbx=1;
        [x0,fval]=fminunc(fopttm,[r(j_row),theta(j_col)]);
        vmax=-fval;
        disp('----------------------------------------------------------------------')
    end
      if abs(vmin-fval)/abs(vmin) > 5E-2 ||  opt_tlbx==0
        if  opt_tlbx==1
            disp('fminunc probably reach a local minimum')
        else
            disp('No optimization toolbox found');
        end
        disp('Maximization by values comparison launched')
        ht=pi/1000;
        theta=ht:ht:pi-ht;
        hr=R/1000;
        r=hr:hr:R-hr;
        y=[];
        for i=1:length(r)
            y(i,:)=fobjtm(r(i),theta);
            Vmin(i)=min(y(i,:));
        end
        vmax=-min(Vmin);
      end

    if m~=0
        vmax=vmax.*(2*ell+1)./(8*pi);
    else
        vmax=vmax.*(2*ell+1)./(4*pi);
    end
    % Square of the maximum valued of the magnetic induction inside the sphere
    EMmax2=vmax;
    % Volume of the mode
    vol=vol/EMmax2;
    disp([' Mode volume (in micron.m^3)= ',num2str(vol)]);
    disp(['CPU time for mode volume computations = ', num2str(cputime-t,4),' sec.'])
end
